package fr.openium.consentium_ui.domain.usecase

import fr.openium.consentium_ui.domain.model.ContentConfigData
import javax.inject.Inject

private const val FALLBACK_LANGUAGE = "en"

internal interface GetConfigTextForLanguageUseCase {
    suspend operator fun invoke(
        language: String = FALLBACK_LANGUAGE,
        configData: ContentConfigData,
    ): GetConfigTextForLanguageUseCaseResponse
}

internal class GetConfigTextForLanguageUseCaseImpl @Inject constructor() :
    GetConfigTextForLanguageUseCase {

    override suspend fun invoke(
        language: String,
        configData: ContentConfigData,
    ): GetConfigTextForLanguageUseCaseResponse {

        return try {

            val canIUseTheTranslation =
                configData.mainTextTranslation.any { it.language == language } &&
                        configData.purposes.all { purposeData ->
                            purposeData.translations.any { it.language == language } &&
                                    purposeData.vendors.all { vendorData ->
                                        vendorData.translations.any { it.language == language }
                                    }
                        }

            val languageToUse = if (canIUseTheTranslation) {
                language
            } else {
                val isThereAGoodFallbackLanguage =
                    configData.mainTextTranslation.any { it.language == FALLBACK_LANGUAGE } &&
                            configData.purposes.all { purposeData ->
                                purposeData.translations.any { it.language == FALLBACK_LANGUAGE } &&
                                        purposeData.vendors.all { vendorData ->
                                            vendorData.translations.any { it.language == FALLBACK_LANGUAGE }
                                        }
                            }
                if (isThereAGoodFallbackLanguage) {
                    FALLBACK_LANGUAGE
                } else {
                    throw Exception()
                }
            }

            val filteredConfigData = configData.copy(
                mainTextTranslation = configData.mainTextTranslation.filter { it.language == languageToUse },
                purposes = configData.purposes.map { purposeData ->
                    purposeData.copy(
                        translations = purposeData.translations.filter { it.language == languageToUse },
                        vendors = purposeData.vendors.map { vendorData ->
                            vendorData.copy(
                                translations = vendorData.translations.filter { it.language == languageToUse }
                            )
                        }
                    )
                }
            )

            if (languageToUse == FALLBACK_LANGUAGE) {
                GetConfigTextForLanguageUseCaseResponse.DefaultLanguage(filteredConfigData)
            } else {
                GetConfigTextForLanguageUseCaseResponse.Success(filteredConfigData)
            }
        } catch (e: Exception) {
            GetConfigTextForLanguageUseCaseResponse.Error
        }
    }

}

internal interface GetConfigTextForLanguageUseCaseResponse {

    data object Error : GetConfigTextForLanguageUseCaseResponse

    data class Success(val configData: ContentConfigData) : GetConfigTextForLanguageUseCaseResponse

    data class DefaultLanguage(val configData: ContentConfigData) :
        GetConfigTextForLanguageUseCaseResponse

}
