package fr.openium.consentium.api

import android.content.Context
import dagger.hilt.android.EntryPointAccessors
import fr.openium.consentium.api.model.ConsentState
import fr.openium.consentium.api.model.Purpose
import fr.openium.consentium.api.model.PurposeChoice
import fr.openium.consentium.api.model.PurposeStatus
import fr.openium.consentium.api.state.FetchConsentiumState
import fr.openium.consentium.api.state.SetConsentiumState
import fr.openium.consentium.domain.di.RepositoryEntryPoint
import fr.openium.consentium.domain.repository.ConsentiumRepository
import fr.openium.consentium.domain.repository.ConsentiumRepositoryGetResponse
import fr.openium.consentium.domain.repository.ConsentiumRepositorySetResponse
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow

class Consentium(
    context: Context,
    val apiKey: String,
    val appId: String,
) {
    private val _fetchConsentState = MutableStateFlow<FetchConsentiumState>(FetchConsentiumState.Idle)
    val fetchConsentState by lazy { _fetchConsentState.asStateFlow() }

    private val _saveConsentState = MutableStateFlow<SetConsentiumState>(SetConsentiumState.Idle)
    val saveConsentState by lazy { _saveConsentState.asStateFlow() }

    private val consentiumRepository: ConsentiumRepository by lazy {
        val appContext = context.applicationContext
        val entryPoint = EntryPointAccessors.fromApplication(appContext, RepositoryEntryPoint::class.java)
        entryPoint.provideConsentiumRepository()
    }

    suspend fun fetchConsents() {
        _fetchConsentState.value = FetchConsentiumState.Loading
        try {
            when (val consentResponse = consentiumRepository.getConsents(apiKey)) {
                ConsentiumRepositoryGetResponse.Error -> _fetchConsentState.value = FetchConsentiumState.Error

                is ConsentiumRepositoryGetResponse.GetConsentsSuccess -> {
                    val areConsentsValid = consentResponse.state == ConsentState.VALID
                    if (areConsentsValid) {
                        _fetchConsentState.value = FetchConsentiumState.Valid(purposes = consentResponse.purposes)
                    } else {
                        _fetchConsentState.value = FetchConsentiumState.Invalid
                    }
                }
            }
        } catch (e: Exception) {
            _fetchConsentState.value = FetchConsentiumState.Error
        }
    }

    suspend fun saveConsents(
        consent: List<PurposeChoice>,
    ) {
        _saveConsentState.emit(SetConsentiumState.Loading)
        try {
            when (consentiumRepository.setConsents(apiKey, consent)) {
                ConsentiumRepositorySetResponse.Error -> {
                    _saveConsentState.emit(SetConsentiumState.Error)
                }

                ConsentiumRepositorySetResponse.SetConsentsSuccess -> {
                    _saveConsentState.emit(SetConsentiumState.Success)
                }
            }
        } catch (e: Exception) {
            _saveConsentState.emit(SetConsentiumState.Error)
        }
    }
}

fun List<Purpose>.checkPurposeState(purposeId: String, onPurposeState: (PurposeStatus) -> Unit) {
    onPurposeState(find { it.identifier == purposeId }?.choice ?: PurposeStatus.REJECTED)
}

fun List<Purpose>.checkPurposeState(purposeId: String): PurposeStatus {
    return find { it.identifier == purposeId }?.choice ?: PurposeStatus.REJECTED
}