package fr.openium.consentium.domain.repository

import fr.openium.consentium.api.adapter.toConsentState
import fr.openium.consentium.api.adapter.toPatchConsentPurposeDTO
import fr.openium.consentium.api.adapter.toPurpose
import fr.openium.consentium.api.model.ConsentState
import fr.openium.consentium.api.model.Purpose
import fr.openium.consentium.api.model.PurposeChoice
import fr.openium.consentium.data.remote.ConsentiumApi
import fr.openium.consentium.data.remote.model.PatchConsent
import fr.openium.consentium.domain.useCase.GetAuthTokenUseCase
import timber.log.Timber
import javax.inject.Inject

internal class ConsentiumRepository @Inject constructor(
    private val consentiumApi: ConsentiumApi,
    private val getAuthTokenUseCase: GetAuthTokenUseCase,
) {

    suspend fun getConsents(apiKey: String): ConsentiumRepositoryGetResponse {
        return try {
            val authToken = getAuthTokenUseCase(apiKey)
            val consentsResponse = consentiumApi.getConsents(authToken)
            val consentsBody = if (consentsResponse.isSuccessful) {
                consentsResponse.body() ?: throw Exception()
            } else {
                throw Exception()
            }

            return ConsentiumRepositoryGetResponse.GetConsentsSuccess(
                state = consentsBody.state?.toConsentState() ?: ConsentState.UNSET,
                purposes = consentsBody.purposes?.map { purpose -> purpose.toPurpose() } ?: emptyList()
            )
        } catch (e: Exception) {
            ConsentiumRepositoryGetResponse.Error
        }
    }

    suspend fun setConsents(
        apiKey: String,
        consents: List<PurposeChoice>,
    ): ConsentiumRepositorySetResponse {
        return try {
            val authToken = getAuthTokenUseCase(apiKey)
            val setConsentResponse = consentiumApi.setConsents(
                token = authToken,
                patchConsent = PatchConsent.PatchConsentPayloadDTO(
                    purposes = consents.map { it.toPatchConsentPurposeDTO() },
                )
            )

            if (setConsentResponse.isSuccessful) {
                ConsentiumRepositorySetResponse.SetConsentsSuccess
            } else {
                ConsentiumRepositorySetResponse.Error
            }
        } catch (e: Exception) {
            Timber.d(e)
            ConsentiumRepositorySetResponse.Error
        }

    }

}

internal interface ConsentiumRepositoryGetResponse {

    data object Error : ConsentiumRepositoryGetResponse

    data class GetConsentsSuccess(
        val state: ConsentState,
        val purposes: List<Purpose>,
    ) : ConsentiumRepositoryGetResponse

}

internal interface ConsentiumRepositorySetResponse {

    data object Error : ConsentiumRepositorySetResponse

    data object SetConsentsSuccess : ConsentiumRepositorySetResponse

}